@@ -38,7 +38,7 @@ pub trait RoutingMsgUtils {
         src: SrcAuthority,
         dst: DstLocation,
         variant: Variant,
-        proof_chain: Option<SecuredLinkedList>,
+        section_key: bls::PublicKey,
     ) -> Result<RoutingMsg, Error>;
 
     /// Creates a message signed using a BLS KeyShare for destination accumulation
@@ -61,7 +61,7 @@ pub trait RoutingMsgUtils {
         node: &Node,
         dst: DstLocation,
         variant: Variant,
-        proof_chain: Option<SecuredLinkedList>,
+        section_key: bls::PublicKey,
     ) -> Result<RoutingMsg>;
 
     /// Creates a signed message from a section.
@@ -69,7 +69,7 @@ pub trait RoutingMsgUtils {
     fn section_src(
         plain: PlainMessage,
         proof: Proof,
-        proof_chain: SecuredLinkedList,
+        section_chain: SecuredLinkedList,
     ) -> Result<RoutingMsg>;
 
     /// Verify this message is properly signed and trusted.
@@ -92,27 +92,13 @@ pub trait RoutingMsgUtils {
     /// Getter
     fn src(&self) -> &SrcAuthority;
 
-    /// Returns the attached proof chain, if any.
-    fn proof_chain(&self) -> Result<&SecuredLinkedList>;
-
-    /// Returns the last key of the attached the proof chain, if any.
-    fn proof_chain_last_key(&self) -> Result<&bls::PublicKey>;
-
-    // Extend the current message proof chain so it starts at `new_first_key` while keeping the
-    // last key (and therefore the signature) intact.
-    // NOTE: This operation doesn't invalidate the signatures because the proof chain is not part of
-    // the signed data.
-    fn extend_proof_chain(
-        self,
-        new_first_key: &bls::PublicKey,
-        full_chain: &SecuredLinkedList,
-    ) -> Result<RoutingMsg, Error>;
-
     fn verify_variant<'a, I: IntoIterator<Item = &'a bls::PublicKey>>(
         &self,
-        proof_chain: Option<&SecuredLinkedList>,
         trusted_keys: I,
     ) -> Result<VerifyStatus>;
+
+    /// Returns an updated message with the provided Section key i.e. known to be latest.
+    fn updated_with_latest_key(&mut self, section_pk: bls::PublicKey);
 }
 
 impl RoutingMsgUtils for RoutingMsg {
@@ -141,8 +127,7 @@ impl RoutingMsgUtils for RoutingMsg {
                     return Err(Error::CreateError(CreateError::FailedSignature));
                 }
 
-                if Some(&proof_share.public_key_set.public_key()) != msg.proof_chain_last_key().ok()
-                {
+                if proof_share.public_key_set.public_key() != msg.section_pk() {
                     error!(
                         "Proof share public key doesn't match proof chain last key: {:?}",
                         msg
@@ -151,17 +136,12 @@ impl RoutingMsgUtils for RoutingMsg {
                 }
             }
             SrcAuthority::Section { proof, .. } => {
-                if let Some(proof_chain) = msg.proof_chain.as_ref() {
-                    if !proof_chain
-                        .last_key()
-                        .verify(&proof.signature, &signed_bytes)
-                    {
-                        error!(
-                            "Failed signature: {:?} (proof chain: {:?})",
-                            msg, proof_chain
-                        );
-                        return Err(Error::CreateError(CreateError::FailedSignature));
-                    }
+                if !msg.section_pk.verify(&proof.signature, &signed_bytes) {
+                    error!(
+                        "Failed signature: {:?} (Section PK: {:?})",
+                        msg, msg.section_pk
+                    );
+                    return Err(Error::CreateError(CreateError::FailedSignature));
                 }
             }
         }
@@ -174,7 +154,7 @@ impl RoutingMsgUtils for RoutingMsg {
         src: SrcAuthority,
         dst: DstLocation,
         variant: Variant,
-        proof_chain: Option<SecuredLinkedList>,
+        section_pk: bls::PublicKey,
     ) -> Result<RoutingMsg, Error> {
         // Create message id from src authority signature
         let id = match &src {
@@ -192,7 +172,7 @@ impl RoutingMsgUtils for RoutingMsg {
             dst,
             aggregation: Aggregation::None,
             variant,
-            proof_chain,
+            section_pk,
         };
 
         Ok(msg)
@@ -204,8 +184,8 @@ impl RoutingMsgUtils for RoutingMsg {
         src_name: XorName,
         dst: DstLocation,
         variant: Variant,
-        proof_chain: SecuredLinkedList,
-    ) -> Result<RoutingMsg, Error> {
+        section_chain: SecuredLinkedList,
+    ) -> Result<Self, Error> {
         let serialized = bincode::serialize(&SignableView {
             dst: &dst,
             variant: &variant,
@@ -222,20 +202,22 @@ impl RoutingMsgUtils for RoutingMsg {
         let src = SrcAuthority::BlsShare {
             src_name,
             proof_share,
+            section_chain: section_chain.clone(),
         };
 
-        RoutingMsg::new_signed(src, dst, variant, Some(proof_chain))
+        Self::new_signed(src, dst, variant, *section_chain.last_key())
     }
 
     /// Converts the message src authority from `BlsShare` to `Section` on successful accumulation.
     /// Returns errors if src is not `BlsShare` or if the proof is invalid.
-    fn into_dst_accumulated(mut self, proof: Proof) -> Result<RoutingMsg> {
-        let (proof_share, src_name) = if let SrcAuthority::BlsShare {
+    fn into_dst_accumulated(mut self, proof: Proof) -> Result<Self> {
+        let (proof_share, src_name, section_chain) = if let SrcAuthority::BlsShare {
             proof_share,
             src_name,
+            section_chain,
         } = &self.src
         {
-            (proof_share.clone(), *src_name)
+            (proof_share.clone(), *src_name, section_chain)
         } else {
             error!("not a message for dst accumulation");
             return Err(Error::InvalidMessage);
@@ -246,8 +228,8 @@ impl RoutingMsgUtils for RoutingMsg {
             return Err(Error::InvalidMessage);
         }
 
-        if Some(&proof.public_key) != self.proof_chain_last_key().ok() {
-            error!("proof public key doesn't match proof chain last key");
+        if proof.public_key != self.section_pk {
+            error!("proof public key doesn't match the attached section PK");
             return Err(Error::InvalidMessage);
         }
 
@@ -257,7 +239,11 @@ impl RoutingMsgUtils for RoutingMsg {
             return Err(Error::FailedSignature);
         }
 
-        self.src = SrcAuthority::Section { proof, src_name };
+        self.src = SrcAuthority::Section {
+            proof,
+            src_name,
+            section_chain: section_chain.clone(),
+        };
 
         Ok(self)
     }
@@ -274,8 +260,8 @@ impl RoutingMsgUtils for RoutingMsg {
         node: &Node,
         dst: DstLocation,
         variant: Variant,
-        proof_chain: Option<SecuredLinkedList>,
-    ) -> Result<RoutingMsg> {
+        section_pk: bls::PublicKey,
+    ) -> Result<Self> {
         let serialized = bincode::serialize(&SignableView {
             dst: &dst,
             variant: &variant,
@@ -288,7 +274,7 @@ impl RoutingMsgUtils for RoutingMsg {
             signature,
         };
 
-        RoutingMsg::new_signed(src, dst, variant, proof_chain)
+        RoutingMsg::new_signed(src, dst, variant, section_pk)
     }
 
     /// Creates a signed message from a section.
@@ -296,16 +282,17 @@ impl RoutingMsgUtils for RoutingMsg {
     fn section_src(
         plain: PlainMessage,
         proof: Proof,
-        proof_chain: SecuredLinkedList,
-    ) -> Result<RoutingMsg> {
-        RoutingMsg::new_signed(
+        section_chain: SecuredLinkedList,
+    ) -> Result<Self> {
+        Self::new_signed(
             SrcAuthority::Section {
                 src_name: plain.src,
                 proof,
+                section_chain: section_chain.clone(),
             },
             plain.dst,
             plain.variant,
-            Some(proof_chain),
+            *section_chain.last_key(),
         )
     }
 
@@ -331,17 +318,15 @@ impl RoutingMsgUtils for RoutingMsg {
                 }
 
                 // Variant-specific verification.
-                self.verify_variant(self.proof_chain.as_ref(), trusted_keys)
+                self.verify_variant(trusted_keys)
             }
-            SrcAuthority::BlsShare { proof_share, .. } => {
+            SrcAuthority::BlsShare {
+                proof_share,
+                section_chain,
+                ..
+            } => {
                 // Proof chain is required for accumulation at destination.
-                let proof_chain = if let Some(proof_chain) = self.proof_chain.as_ref() {
-                    proof_chain
-                } else {
-                    return Err(Error::InvalidMessage);
-                };
-
-                if proof_share.public_key_set.public_key() != *proof_chain.last_key() {
+                if proof_share.public_key_set.public_key() != self.section_pk {
                     return Err(Error::InvalidMessage);
                 }
 
@@ -349,25 +334,23 @@ impl RoutingMsgUtils for RoutingMsg {
                     return Err(Error::FailedSignature);
                 }
 
-                if proof_chain.check_trust(trusted_keys) {
+                if section_chain.check_trust(trusted_keys) {
                     Ok(VerifyStatus::Full)
                 } else {
                     Ok(VerifyStatus::Unknown)
                 }
             }
-            SrcAuthority::Section { proof, .. } => {
+            SrcAuthority::Section {
+                proof,
+                section_chain,
+                ..
+            } => {
                 // Proof chain is required for section-src messages.
-                let proof_chain = if let Some(proof_chain) = self.proof_chain.as_ref() {
-                    proof_chain
-                } else {
-                    return Err(Error::InvalidMessage);
-                };
-
-                if !proof_chain.last_key().verify(&proof.signature, &bytes) {
+                if !self.section_pk.verify(&proof.signature, &bytes) {
                     return Err(Error::FailedSignature);
                 }
 
-                if proof_chain.check_trust(trusted_keys) {
+                if section_chain.check_trust(trusted_keys) {
                     Ok(VerifyStatus::Full)
                 } else {
                     Ok(VerifyStatus::Unknown)
@@ -405,49 +388,7 @@ impl RoutingMsgUtils for RoutingMsg {
         &self.src
     }
 
-    /// Returns the attached proof chain, if any.
-    fn proof_chain(&self) -> Result<&SecuredLinkedList> {
-        self.proof_chain.as_ref().ok_or(Error::InvalidMessage)
-    }
-
-    /// Returns the last key of the attached the proof chain, if any.
-    fn proof_chain_last_key(&self) -> Result<&bls::PublicKey> {
-        self.proof_chain().map(|proof_chain| proof_chain.last_key())
-    }
-
-    // Extend the current message proof chain so it starts at `new_first_key` while keeping the
-    // last key (and therefore the signature) intact.
-    // NOTE: This operation doesn't invalidate the signatures because the proof chain is not part of
-    // the signed data.
-    fn extend_proof_chain(
-        mut self,
-        new_first_key: &bls::PublicKey,
-        full_chain: &SecuredLinkedList,
-    ) -> Result<RoutingMsg, Error> {
-        let proof_chain = self
-            .proof_chain
-            .as_mut()
-            .ok_or(ExtendProofChainError::NoProofChain)?;
-
-        *proof_chain = match proof_chain.extend(new_first_key, full_chain) {
-            Ok(chain) => chain,
-            Err(SecuredLinkedListError::InvalidOperation) => {
-                // This means the tip of the proof chain is not reachable from `new_first_key`.
-                // Extend it from the root key of the full chain instead as that should be the
-                // genesis key which is implicitly trusted.
-                proof_chain.extend(full_chain.root_key(), full_chain)?
-            }
-            Err(error) => return Err(error.into()),
-        };
-
-        RoutingMsg::new_signed(self.src, self.dst, self.variant, self.proof_chain)
-    }
-
-    fn verify_variant<'a, I>(
-        &self,
-        proof_chain: Option<&SecuredLinkedList>,
-        trusted_keys: I,
-    ) -> Result<VerifyStatus>
+    fn verify_variant<'a, I>(&self, trusted_keys: I) -> Result<VerifyStatus>
     where
         I: IntoIterator<Item = &'a bls::PublicKey>,
     {
@@ -455,20 +396,23 @@ impl RoutingMsgUtils for RoutingMsg {
             Variant::NodeApproval {
                 section_auth,
                 member_info,
+                section_chain,
                 ..
             } => {
-                let proof_chain = proof_chain.ok_or(Error::InvalidMessage)?;
-
-                if !section_auth.verify(proof_chain) {
+                if !section_auth.verify(section_chain) {
                     return Err(Error::InvalidMessage);
                 }
 
-                if !member_info.verify(proof_chain) {
+                if !member_info.verify(section_chain) {
                     return Err(Error::InvalidMessage);
                 }
 
-                proof_chain
+                section_chain
             }
+            Variant::SectionKnowledge {
+                src_info: (_, chain),
+                ..
+            } => chain,
             Variant::Sync { section, .. } => section.chain(),
             _ => return Ok(VerifyStatus::Full),
         };
@@ -479,6 +423,10 @@ impl RoutingMsgUtils for RoutingMsg {
             Ok(VerifyStatus::Unknown)
         }
     }
+
+    fn updated_with_latest_key(&mut self, section_pk: bls::PublicKey) {
+        self.section_pk = section_pk
+    }
 }
 
 #[derive(Eq, PartialEq, Debug)]
@@ -528,65 +476,3 @@ pub struct SignableView<'a> {
     pub dst: &'a DstLocation,
     pub variant: &'a Variant,
 }
-
-#[cfg(test)]
-mod tests {
-    use super::*;
-    use crate::{
-        agreement, crypto,
-        peer::PeerUtils,
-        section::{self, test_utils::gen_addr, MemberInfoUtils},
-        MIN_ADULT_AGE,
-    };
-    use anyhow::Result;
-    use sn_messaging::node::{MemberInfo, Peer};
-    use std::iter;
-    use xor_name::Prefix;
-
-    #[test]
-    fn extend_proof_chain() -> Result<()> {
-        let node = Node::new(
-            crypto::gen_keypair(&Prefix::default().range_inclusive(), MIN_ADULT_AGE),
-            gen_addr(),
-        );
-
-        let sk0 = bls::SecretKey::random();
-        let pk0 = sk0.public_key();
-
-        let sk1 = bls::SecretKey::random();
-        let pk1 = sk1.public_key();
-
-        let mut full_proof_chain = SecuredLinkedList::new(pk0);
-        let pk1_sig = sk0.sign(&bincode::serialize(&pk1)?);
-        let _ = full_proof_chain.insert(&pk0, pk1, pk1_sig);
-
-        let (section_auth, _, _) =
-            section::test_utils::gen_section_authority_provider(Prefix::default(), 3);
-        let section_auth = agreement::test_utils::proven(&sk1, section_auth)?;
-
-        let peer = Peer::new(rand::random(), gen_addr());
-        let member_info = MemberInfo::joined(peer);
-        let member_info = agreement::test_utils::proven(&sk1, member_info)?;
-
-        let variant = Variant::NodeApproval {
-            genesis_key: pk0,
-            section_auth,
-            member_info,
-        };
-        let message = RoutingMsg::single_src(
-            &node,
-            DstLocation::DirectAndUnrouted,
-            variant,
-            Some(full_proof_chain.truncate(1)),
-        )?;
-
-        assert_eq!(message.verify(iter::once(&pk1))?, VerifyStatus::Full);
-        assert_eq!(message.verify(iter::once(&pk0))?, VerifyStatus::Unknown);
-
-        let message = message.extend_proof_chain(&pk0, &full_proof_chain)?;
-
-        assert_eq!(message.verify(iter::once(&pk0))?, VerifyStatus::Full);
-
-        Ok(())
-    }
-}
