@@ -446,8 +446,12 @@ impl<'a> State<'a> {
         info!("Sending {:?} to {:?}", join_request, recipients);
 
         let variant = Variant::JoinRequest(Box::new(join_request));
-        let message =
-            RoutingMsg::single_src(&self.node, DstLocation::DirectAndUnrouted, variant, None)?;
+        let message = RoutingMsg::single_src(
+            &self.node,
+            DstLocation::DirectAndUnrouted,
+            variant,
+            section_key,
+        )?;
 
         let _ = self
             .send_tx
@@ -528,6 +532,7 @@ impl<'a> State<'a> {
                     genesis_key,
                     section_auth,
                     member_info,
+                    section_chain,
                 } => {
                     if member_info.value.peer.name() != &self.node.name() {
                         trace!("Ignore NodeApproval not for us");
@@ -551,8 +556,6 @@ impl<'a> State<'a> {
                         continue;
                     }
 
-                    let section_chain = message.proof_chain()?.clone();
-
                     trace!(
                         "This node has been approved to join the network at {:?}!",
                         section_auth.value.prefix,
@@ -562,7 +565,7 @@ impl<'a> State<'a> {
                         JoinResponse::Approval {
                             section_auth: section_auth.clone(),
                             genesis_key: *genesis_key,
-                            section_chain,
+                            section_chain: section_chain.clone(),
                         },
                         sender,
                         dest_info,
@@ -792,10 +795,11 @@ mod tests {
                 DstLocation::DirectAndUnrouted,
                 Variant::NodeApproval {
                     genesis_key: pk,
-                    section_auth,
+                    section_auth: section_auth.clone(),
                     member_info,
+                    section_chain: proof_chain,
                 },
-                Some(proof_chain),
+                section_auth.value.section_key,
             )?;
 
             recv_tx.try_send((
@@ -1243,7 +1247,7 @@ mod tests {
                     section_auth: gen_section_authority_provider(bad_prefix, ELDER_SIZE).0,
                     section_key: bls::SecretKey::random().public_key(),
                 },
-                None,
+                section_key,
             )?;
 
             recv_tx.try_send((
@@ -1266,7 +1270,7 @@ mod tests {
                     section_auth: gen_section_authority_provider(good_prefix, ELDER_SIZE).0,
                     section_key: bls::SecretKey::random().public_key(),
                 },
-                None,
+                section_key,
             )?;
 
             recv_tx.try_send((
