@@ -31,11 +31,7 @@ use xor_name::XorName;
 // RoutingMsg sending
 impl Core {
     // Send NodeApproval to a joining node which makes them a section member
-    pub(crate) fn send_node_approval(
-        &self,
-        member_info: Proven<MemberInfo>,
-        their_knowledge: Option<bls::PublicKey>,
-    ) -> Result<Command> {
+    pub(crate) fn send_node_approval(&self, member_info: Proven<MemberInfo>) -> Result<Command> {
         info!(
             "Our section with {:?} has approved peer {:?}.",
             self.section.prefix(),
@@ -45,25 +41,18 @@ impl Core {
         let addr = *member_info.value.peer.addr();
         let name = *member_info.value.peer.name();
 
-        // Attach proof chain that includes the key the approved node knows (if any), the key its
-        // `MemberInfo` is signed with and the last key of our section chain.
-        let proof_chain = self.section.chain().minimize(
-            iter::once(self.section.chain().last_key())
-                .chain(their_knowledge.as_ref())
-                .chain(iter::once(&member_info.proof.public_key)),
-        )?;
-
         let variant = Variant::NodeApproval {
             genesis_key: *self.section.genesis_key(),
             section_auth: self.section.proven_authority_provider().clone(),
             member_info,
+            section_chain: self.section.chain().clone(),
         };
 
         let message = RoutingMsg::single_src(
             &self.node,
             DstLocation::DirectAndUnrouted,
             variant,
-            Some(proof_chain),
+            self.section.authority_provider().section_key,
         )?;
 
         Ok(Command::send_message_to_node(
@@ -80,8 +69,12 @@ impl Core {
         let send = |variant, recipients: Vec<(XorName, SocketAddr)>| -> Result<_> {
             trace!("Send {:?} to {:?}", variant, recipients);
 
-            let message =
-                RoutingMsg::single_src(&self.node, DstLocation::DirectAndUnrouted, variant, None)?;
+            let message = RoutingMsg::single_src(
+                &self.node,
+                DstLocation::DirectAndUnrouted,
+                variant,
+                self.section.authority_provider().section_key,
+            )?;
             let dest_info = DestInfo {
                 dest: XorName::random(),
                 dest_section_pk: *self.section.chain().last_key(),
@@ -122,8 +115,12 @@ impl Core {
         let send = |variant, recipients: Vec<_>| -> Result<_> {
             trace!("Send {:?} to {:?}", variant, recipients);
 
-            let message =
-                RoutingMsg::single_src(&self.node, DstLocation::DirectAndUnrouted, variant, None)?;
+            let message = RoutingMsg::single_src(
+                &self.node,
+                DstLocation::DirectAndUnrouted,
+                variant,
+                self.section.authority_provider().section_key,
+            )?;
 
             Ok(Command::send_message_to_nodes(
                 recipients.clone(),
@@ -159,23 +156,11 @@ impl Core {
         recipient: &Peer,
         details: RelocateDetails,
     ) -> Result<Vec<Command>> {
-        // We need to construct a proof that would be trusted by the destination section.
-        let known_key = self
-            .network
-            .key_by_name(&details.destination)
-            .unwrap_or_else(|_| self.section.chain().root_key());
-
         let src = details.pub_id;
         let dst = DstLocation::Node(details.pub_id);
         let variant = Variant::Relocate(details);
 
-        self.send_message_for_dst_accumulation(
-            src,
-            dst,
-            variant,
-            Some(known_key),
-            slice::from_ref(recipient),
-        )
+        self.send_message_for_dst_accumulation(src, dst, variant, slice::from_ref(recipient))
     }
 
     pub(crate) fn send_relocate_promise(
@@ -190,7 +175,7 @@ impl Core {
         let dst = DstLocation::Section(promise.name);
         let variant = Variant::RelocatePromise(promise);
 
-        self.send_message_for_dst_accumulation(src, dst, variant, None, slice::from_ref(recipient))
+        self.send_message_for_dst_accumulation(src, dst, variant, slice::from_ref(recipient))
     }
 
     pub(crate) fn return_relocate_promise(&self) -> Option<Command> {
@@ -233,7 +218,6 @@ impl Core {
             src_prefix.name(),
             DstLocation::DirectAndUnrouted,
             variant,
-            None,
             recipients,
         )
     }
@@ -274,10 +258,8 @@ impl Core {
         src: XorName,
         dst: DstLocation,
         variant: Variant,
-        additional_proof_chain_key: Option<&bls::PublicKey>,
         recipients: &[Peer],
     ) -> Result<Vec<Command>> {
-        let proof_chain = self.create_proof_chain(additional_proof_chain_key)?;
         let key_share = self.section_keys_provider.key_share().map_err(|err| {
             trace!(
                 "Can't create message {:?} for accumulation at dst {:?}: {}",
@@ -287,7 +269,13 @@ impl Core {
             );
             err
         })?;
-        let message = RoutingMsg::for_dst_accumulation(key_share, src, dst, variant, proof_chain)?;
+        let message = RoutingMsg::for_dst_accumulation(
+            key_share,
+            src,
+            dst,
+            variant,
+            self.section.chain().clone(),
+        )?;
 
         trace!(
             "Send {:?} for accumulation at dst to {:?}",
@@ -374,8 +362,12 @@ impl Core {
         variant: Variant,
         dst_pk: bls::PublicKey,
     ) -> Result<Command> {
-        let message =
-            RoutingMsg::single_src(&self.node, DstLocation::DirectAndUnrouted, variant, None)?;
+        let message = RoutingMsg::single_src(
+            &self.node,
+            DstLocation::DirectAndUnrouted,
+            variant,
+            self.section.authority_provider().section_key,
+        )?;
         Ok(Command::send_message_to_node(
             recipient,
             message,
